@@ -28,6 +28,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/wait.h>
+#include <time.h>
 #include <ldap.h> /* fixme: remove it */
 #include <pth.h>
 #include <ksba.h> 
@@ -64,7 +65,7 @@ struct cert_fetch_context_s
 };
 
 
-/* To keep track of the LDAp wrapper state we use this structure.  */
+/* To keep track of the LDAP wrapper state we use this structure.  */
 struct wrapper_context_s
 {
   struct wrapper_context_s *next;
@@ -94,6 +95,9 @@ struct wrapper_context_s
    finished processes. */
 static struct wrapper_context_s *wrapper_list;
 
+/* We need to know whether we are shutting down the process */
+static int shutting_down;
+
 
 /* Prototypes.  */
 static gpg_error_t read_buffer (ksba_reader_t reader,
@@ -342,10 +346,10 @@ ldap_wrapper_thread (void *dummy)
 
       /* Use a separate loop to check whether ready marked wrappers
          may be removed.  We may only do so if the ksba reader object
-         is not anymore in use. */
+         is not anymore in use or we are in shutdown state.  */
      again:
       for (ctx_prev=NULL, ctx=wrapper_list; ctx; ctx_prev=ctx, ctx=ctx->next)
-        if (ctx->ready && !ctx->reader)
+        if (ctx->ready && (!ctx->reader || shutting_down))
           {
             if (ctx_prev)
               ctx_prev->next = ctx->next;
@@ -367,6 +371,7 @@ ldap_wrapper_thread (void *dummy)
 void
 ldap_wrapper_wait_connections ()
 {
+  shutting_down = 1;
   while (wrapper_list)
     pth_yield (NULL);
 }
@@ -405,7 +410,7 @@ ldap_wrapper_release_context (ksba_reader_t reader)
       }
 }
 
-/* Cleanup all resources hel by the connection associated with
+/* Cleanup all resources held by the connection associated with
    CTRL.  This is used after a cancel to kill running wrappers.  */
 void
 ldap_wrapper_connection_cleanup (ctrl_t ctrl)
@@ -707,7 +712,7 @@ ldap_wrapper (ctrl_t ctrl, ksba_reader_t *reader, const char *argv[])
     log_info (_("ldap wrapper %d started\n"), (int)ctx->pid);
 
   /* Need to wait for the first byte so we are able to detect an empty
-     output and not let the consumer se an EOF without further error
+     output and not let the consumer see an EOF without further error
      indications.  The CRL loading logic assumes that after return
      from this function, a failed search (e.g. host not found ) is
      indicated right away. */
@@ -1247,7 +1252,7 @@ start_cert_fetch_ldap (ctrl_t ctrl, cert_fetch_context_t *context,
       *context = NULL;
     }
 
-  return 0;
+  return err;
 }
 
 
@@ -1272,7 +1277,7 @@ read_buffer (ksba_reader_t reader, unsigned char *buffer, size_t count)
 
 /* Fetch the next certificate. Return 0 on success, GPG_ERR_EOF if no
    (more) certificates are available or any other error
-   code. GPG_ERR_TRUNCATED may be returned to indicate tha the result
+   code. GPG_ERR_TRUNCATED may be returned to indicate that the result
    has been truncated. */
 gpg_error_t
 fetch_next_cert_ldap (cert_fetch_context_t context,
@@ -1301,7 +1306,7 @@ fetch_next_cert_ldap (cert_fetch_context_t context,
        
           if (is_cms)
             {
-              /* The certificate needs to be parsed from CMS data.  Read */
+              /* The certificate needs to be parsed from CMS data. */
               ksba_cms_t cms;
               ksba_stop_reason_t stopreason;
               int i;
