@@ -1,6 +1,6 @@
 /* dirmngr.c - LDAP access
  *	Copyright (C) 2002 Klarälvdalens Datakonsult AB
- *      Copyright (C) 2003, 2004 g10 Code GmbH
+ *      Copyright (C) 2003, 2004, 2005 g10 Code GmbH
  *
  * This file is part of DirMngr.
  *
@@ -39,6 +39,7 @@
 #include "ocsp.h"
 #include "certcache.h"
 #include "validate.h"
+#include "misc.h"
 
 /* To avoid DoS attacks we limit the size of a certificate to
    something reasonable. */
@@ -128,7 +129,8 @@ do_get_cert_local (ctrl_t ctrl, const char *name, const char *command)
   xfree (buf);
   if (rc)
     {
-      log_error (_("assuan_inquire failed: %s\n"), assuan_strerror (rc));
+      log_error (_("assuan_inquire(%s) failed: %s\n"),
+                 command, assuan_strerror (rc));
       return NULL;
     }
   
@@ -163,32 +165,106 @@ do_get_cert_local (ctrl_t ctrl, const char *name, const char *command)
 ksba_cert_t 
 get_cert_local (ctrl_t ctrl, const char *name)
 {
-  if (!ctrl || !ctrl->server_local->assuan_ctx)
+  if (!ctrl || !ctrl->server_local || !ctrl->server_local->assuan_ctx)
     {
-      log_debug ("get_cert_local called w/o context\n");
+      if (opt.debug)
+        log_debug ("get_cert_local called w/o context\n");
       return NULL;
     }
   return do_get_cert_local (ctrl, name, "SENDCERT");
 
 }
-
+       
 /* Ask back to return the issuing certificate for name, given as a
    regular gpgsm certificate indentificates (e.g. fingerprint or one
    of the other methods).  Alternatively, NULL may be used for NAME to
    return thecurrent target certificate. Either return the certificate
    in a KSBA object or NULL if it is not available.
+   
 */
 ksba_cert_t 
 get_issuing_cert_local (ctrl_t ctrl, const char *name)
 {
-  if (!ctrl || !ctrl->server_local->assuan_ctx)
+  if (!ctrl || !ctrl->server_local || !ctrl->server_local->assuan_ctx)
     {
-      log_debug ("get_issuing_cert_local called w/o context\n");
+      if (opt.debug)
+        log_debug ("get_issuing_cert_local called w/o context\n");
       return NULL;
     }
   return do_get_cert_local (ctrl, name, "SENDISSUERCERT");
 }
 
+/* Ask back to return a certificate with subject NAME and a
+   subjectKeyIdentifier of KEYID. */
+ksba_cert_t 
+get_cert_local_ski (ctrl_t ctrl, const char *name, ksba_sexp_t keyid)
+{
+  unsigned char *value;
+  size_t valuelen; 
+  int rc;
+  char *buf;
+  ksba_cert_t cert;
+  char *hexkeyid;
+
+  if (!ctrl || !ctrl->server_local || !ctrl->server_local->assuan_ctx)
+    {
+      if (opt.debug)
+        log_debug ("get_cert_local_ski called w/o context\n");
+      return NULL;
+    }
+  if (!name || !keyid)
+    {
+      log_debug ("get_cert_local_ski called with insufficient arguments\n");
+      return NULL;
+    }
+
+  hexkeyid = serial_hex (keyid);
+  if (!hexkeyid)
+    {
+      log_debug ("serial_hex() failed\n");
+      return NULL;
+    }
+
+  buf = xtrymalloc (15 + strlen (hexkeyid) + 2 + strlen(name) + 1);
+  if (!buf)
+    {
+
+      log_error ("can't allocate enough memory: %s\n", strerror (errno));
+      xfree (hexkeyid);
+      return NULL;
+    }
+  strcpy (stpcpy (stpcpy (stpcpy (buf, "SENDCERT_SKI "), hexkeyid)," /"),name);
+  xfree (hexkeyid);
+
+  rc = assuan_inquire (ctrl->server_local->assuan_ctx, buf,
+                       &value, &valuelen, MAX_CERT_LENGTH);
+  xfree (buf);
+  if (rc)
+    {
+      log_error (_("assuan_inquire(%s) failed: %s\n"), "SENDCERT_SKI",
+                 assuan_strerror (rc));
+      return NULL;
+    }
+  
+  if (!valuelen)
+    {
+      xfree (value);
+      return NULL;
+    }
+
+  rc = ksba_cert_new (&cert);
+  if (!rc)
+    {
+      rc = ksba_cert_init_from_mem (cert, value, valuelen);
+      if (rc)
+        {
+          ksba_cert_release (cert);
+          cert = NULL;
+        }
+    }
+  xfree (value);
+  return cert;
+}
 
 
 
@@ -573,6 +649,7 @@ cmd_lookup (assuan_context_t ctx, char *line)
   size_t valuelen; 
   ldap_server_t ldapserver;
   cert_fetch_context_t fetch_context;
+  int any_no_data = 0;
 
   /* Break the line down into an STRLIST */
   for (p=line; *p; line = p)
@@ -610,6 +687,14 @@ cmd_lookup (assuan_context_t ctx, char *line)
 
       /* Fetch certificates matching pattern */
       err = start_cert_fetch (ctrl, &fetch_context, list, ldapserver);
+      if ( gpg_err_code (err) == GPG_ERR_NO_DATA )
+        {
+          if (DBG_LOOKUP)
+            log_debug ("cmd_lookup: no data\n");
+          err = 0;
+          any_no_data = 1;
+          continue;
+        }
       if (err)
         {
           log_error (_("start_cert_fetch failed: %s\n"), gpg_strerror (err));
@@ -621,6 +706,12 @@ cmd_lookup (assuan_context_t ctx, char *line)
         {
           xfree (value); value = NULL;
           err = fetch_next_cert (fetch_context, &value, &valuelen);
+          if (gpg_err_code (err) == GPG_ERR_NO_DATA )
+            {
+              err = 0;
+              any_no_data = 1;
+              break; /* Ready. */
+            }
           if (gpg_err_code (err) == GPG_ERR_TRUNCATED)
             {
               truncated = 1;
@@ -682,6 +773,9 @@ cmd_lookup (assuan_context_t ctx, char *line)
       assuan_write_status (ctx, "TRUNCATED", str);    
     }
 
+  if (!err && !count && any_no_data)
+    err = gpg_error (GPG_ERR_NO_DATA);
+
  leave:
   if (err)
     log_error (_("command %s failed: %s\n"), "LOOKUP", gpg_strerror (err));
